@@ -8,6 +8,7 @@ extern "C" {
 #include <stdlib.h>
 #include <stdarg.h>
 #include <stdint.h>
+#include <stdio.h>
 #include <stdbool.h>
 
 #include <sys/time.h>
@@ -53,6 +54,9 @@ TickitPen *tickit_pen_new_attrs(TickitPenAttr attr, ...);
 TickitPen *tickit_pen_clone(const TickitPen *orig);
 void       tickit_pen_destroy(TickitPen *pen);
 
+TickitPen *tickit_pen_ref(TickitPen *pen);
+void       tickit_pen_unref(TickitPen *pen);
+
 bool tickit_pen_has_attr(const TickitPen *pen, TickitPenAttr attr);
 bool tickit_pen_is_nonempty(const TickitPen *pen);
 bool tickit_pen_nondefault_attr(const TickitPen *pen, TickitPenAttr attr);
@@ -77,9 +81,9 @@ bool tickit_pen_equiv(const TickitPen *a, const TickitPen *b);
 void tickit_pen_copy_attr(TickitPen *dst, const TickitPen *src, TickitPenAttr attr);
 void tickit_pen_copy(TickitPen *dst, const TickitPen *src, bool overwrite);
 
-typedef int TickitPenEventFn(TickitPen *tt, TickitEventType ev, void *info, void *data);
+typedef int TickitPenEventFn(TickitPen *tt, TickitEventType ev, void *info, void *user);
 
-int  tickit_pen_bind_event(TickitPen *tt, TickitEventType ev, TickitPenEventFn *fn, void *data);
+int  tickit_pen_bind_event(TickitPen *tt, TickitEventType ev, TickitPenEventFn *fn, void *user);
 void tickit_pen_unbind_event_id(TickitPen *tt, int id);
 
 TickitPenAttrType tickit_pen_attrtype(TickitPenAttr attr);
@@ -133,6 +137,8 @@ size_t tickit_rectset_get_rects(const TickitRectSet *trs, TickitRect rects[], si
 void tickit_rectset_add(TickitRectSet *trs, const TickitRect *rect);
 void tickit_rectset_subtract(TickitRectSet *trs, const TickitRect *rect);
 
+void tickit_rectset_translate(TickitRectSet *trs, int downward, int rightward);
+
 bool tickit_rectset_intersects(const TickitRectSet *trs, const TickitRect *rect);
 bool tickit_rectset_contains(const TickitRectSet *trs, const TickitRect *rect);
 
@@ -147,6 +153,8 @@ TickitTerm *tickit_term_new(void);
 TickitTerm *tickit_term_new_for_termtype(const char *termtype);
 void tickit_term_destroy(TickitTerm *tt);
 
+TickitTerm *tickit_term_open_stdio(void);
+
 const char *tickit_term_get_termtype(TickitTerm *tt);
 
 void tickit_term_set_output_fd(TickitTerm *tt, int fd);
@@ -180,9 +188,11 @@ void tickit_term_get_size(const TickitTerm *tt, int *lines, int *cols);
 void tickit_term_set_size(TickitTerm *tt, int lines, int cols);
 void tickit_term_refresh_size(TickitTerm *tt);
 
-typedef int TickitTermEventFn(TickitTerm *tt, TickitEventType ev, void *info, void *data);
+void tickit_term_observe_sigwinch(TickitTerm *tt, bool observe);
 
-int  tickit_term_bind_event(TickitTerm *tt, TickitEventType ev, TickitTermEventFn *fn, void *data);
+typedef int TickitTermEventFn(TickitTerm *tt, TickitEventType ev, void *info, void *user);
+
+int  tickit_term_bind_event(TickitTerm *tt, TickitEventType ev, TickitTermEventFn *fn, void *user);
 void tickit_term_unbind_event_id(TickitTerm *tt, int id);
 
 void tickit_term_print(TickitTerm *tt, const char *str);
@@ -221,10 +231,10 @@ typedef enum {
 } TickitTermMouseMode;
 
 typedef enum {
-  TICKIT_TERM_CURSORSHAPE_BLOCK = 1,
-  TICKIT_TERM_CURSORSHAPE_UNDER,
-  TICKIT_TERM_CURSORSHAPE_LEFT_BAR,
-} TickitTermCursorShape;
+  TICKIT_CURSORSHAPE_BLOCK = 1,
+  TICKIT_CURSORSHAPE_UNDER,
+  TICKIT_CURSORSHAPE_LEFT_BAR,
+} TickitCursorShape;
 
 bool tickit_term_getctl_int(TickitTerm *tt, TickitTermCtl ctl, int *value);
 bool tickit_term_setctl_int(TickitTerm *tt, TickitTermCtl ctl, int value);
@@ -310,7 +320,7 @@ void tickit_renderbuffer_get_cursorpos(const TickitRenderBuffer *rb, int *line,
 void tickit_renderbuffer_goto(TickitRenderBuffer *rb, int line, int col);
 void tickit_renderbuffer_ungoto(TickitRenderBuffer *rb);
 
-void tickit_renderbuffer_setpen(TickitRenderBuffer *rb, TickitPen *pen);
+void tickit_renderbuffer_setpen(TickitRenderBuffer *rb, const TickitPen *pen);
 
 void tickit_renderbuffer_reset(TickitRenderBuffer *rb);
 
@@ -321,17 +331,21 @@ void tickit_renderbuffer_restore(TickitRenderBuffer *rb);
 void tickit_renderbuffer_skip_at(TickitRenderBuffer *rb, int line, int col, int cols);
 void tickit_renderbuffer_skip(TickitRenderBuffer *rb, int cols);
 void tickit_renderbuffer_skip_to(TickitRenderBuffer *rb, int col);
-int tickit_renderbuffer_text_at(TickitRenderBuffer *rb, int line, int col, char *text, TickitPen *pen);
-int tickit_renderbuffer_textn_at(TickitRenderBuffer *rb, int line, int col, char *text, size_t len, TickitPen *pen);
-int tickit_renderbuffer_text(TickitRenderBuffer *rb, char *text, TickitPen *pen);
-int tickit_renderbuffer_textn(TickitRenderBuffer *rb, char *text, size_t len, TickitPen *pen);
-void tickit_renderbuffer_erase_at(TickitRenderBuffer *rb, int line, int col, int cols, TickitPen *pen);
-void tickit_renderbuffer_erase(TickitRenderBuffer *rb, int cols, TickitPen *pen);
-void tickit_renderbuffer_erase_to(TickitRenderBuffer *rb, int col, TickitPen *pen);
-void tickit_renderbuffer_eraserect(TickitRenderBuffer *rb, TickitRect *rect, TickitPen *pen);
-void tickit_renderbuffer_clear(TickitRenderBuffer *rb, TickitPen *pen);
-void tickit_renderbuffer_char_at(TickitRenderBuffer *rb, int line, int col, long codepoint, TickitPen *pen);
-void tickit_renderbuffer_char(TickitRenderBuffer *rb, long codepoint, TickitPen *pen);
+int tickit_renderbuffer_text_at(TickitRenderBuffer *rb, int line, int col, const char *text);
+int tickit_renderbuffer_textn_at(TickitRenderBuffer *rb, int line, int col, const char *text, size_t len);
+int tickit_renderbuffer_text(TickitRenderBuffer *rb, const char *text);
+int tickit_renderbuffer_textn(TickitRenderBuffer *rb, const char *text, size_t len);
+int tickit_renderbuffer_textf_at(TickitRenderBuffer *rb, int line, int col, const char *fmt, ...);
+int tickit_renderbuffer_vtextf_at(TickitRenderBuffer *rb, int line, int col, const char *fmt, va_list args);
+int tickit_renderbuffer_textf(TickitRenderBuffer *rb, const char *fmt, ...);
+int tickit_renderbuffer_vtextf(TickitRenderBuffer *rb, const char *fmt, va_list args);
+void tickit_renderbuffer_erase_at(TickitRenderBuffer *rb, int line, int col, int cols);
+void tickit_renderbuffer_erase(TickitRenderBuffer *rb, int cols);
+void tickit_renderbuffer_erase_to(TickitRenderBuffer *rb, int col);
+void tickit_renderbuffer_eraserect(TickitRenderBuffer *rb, TickitRect *rect);
+void tickit_renderbuffer_clear(TickitRenderBuffer *rb);
+void tickit_renderbuffer_char_at(TickitRenderBuffer *rb, int line, int col, long codepoint);
+void tickit_renderbuffer_char(TickitRenderBuffer *rb, long codepoint);
 
 typedef enum {
   TICKIT_LINE_SINGLE = 1,
@@ -346,9 +360,9 @@ typedef enum {
 } TickitLineCaps;
 
 void tickit_renderbuffer_hline_at(TickitRenderBuffer *rb, int line, int startcol, int endcol,
-    TickitLineStyle style, TickitPen *pen, TickitLineCaps caps);
+    TickitLineStyle style, TickitLineCaps caps);
 void tickit_renderbuffer_vline_at(TickitRenderBuffer *rb, int startline, int endline, int col,
-    TickitLineStyle style, TickitPen *pen, TickitLineCaps caps);
+    TickitLineStyle style, TickitLineCaps caps);
 
 void tickit_renderbuffer_flush_to_term(TickitRenderBuffer *rb, TickitTerm *tt);
 
@@ -386,6 +400,84 @@ size_t tickit_renderbuffer_get_span(TickitRenderBuffer *rb, int line, int startc
  */
 
 typedef struct TickitWindow TickitWindow;
+typedef int TickitWindowEventFn(TickitWindow *win, TickitEventType ev, void *info, void *user);
+
+typedef enum {
+  TICKIT_WINDOW_HIDDEN      = 1<<0,
+  TICKIT_WINDOW_LOWEST      = 1<<1,
+  TICKIT_WINDOW_ROOT_PARENT = 1<<2,
+  TICKIT_WINDOW_STEAL_INPUT = 1<<3,
+
+  // Composite flag
+  TICKIT_WINDOW_POPUP = TICKIT_WINDOW_ROOT_PARENT|TICKIT_WINDOW_STEAL_INPUT,
+} TickitWindowFlags;
+
+TickitWindow *tickit_window_new_root(TickitTerm *term);
+TickitWindow *tickit_window_new(TickitWindow *parent, TickitRect rect, TickitWindowFlags flags);
+
+TickitWindow *tickit_window_parent(const TickitWindow *win);
+TickitWindow *tickit_window_root(const TickitWindow *win);
+
+void tickit_window_destroy(TickitWindow *win);
+
+int  tickit_window_bind_event(TickitWindow *win, TickitEventType ev, TickitWindowEventFn *fn, void *user);
+void tickit_window_unbind_event_id(TickitWindow *win, int id);
+
+void tickit_window_raise(TickitWindow *win);
+void tickit_window_raise_to_front(TickitWindow *win);
+void tickit_window_lower(TickitWindow *win);
+void tickit_window_lower_to_back(TickitWindow *win);
+
+void tickit_window_show(TickitWindow *win);
+void tickit_window_hide(TickitWindow *win);
+bool tickit_window_is_visible(TickitWindow *win);
+
+TickitRect tickit_window_get_geometry(const TickitWindow *win);
+TickitRect tickit_window_get_abs_geometry(const TickitWindow *win);
+
+#define tickit_window_top(win)   (tickit_window_get_geometry(win)).top
+#define tickit_window_left(win)  (tickit_window_get_geometry(win)).left
+#define tickit_window_lines(win) (tickit_window_get_geometry(win)).lines
+#define tickit_window_cols(win)  (tickit_window_get_geometry(win)).cols
+
+int tickit_window_bottom(const TickitWindow *win);
+int tickit_window_right(const TickitWindow *win);
+
+void tickit_window_resize(TickitWindow *win, int lines, int cols);
+void tickit_window_reposition(TickitWindow *win, int top, int left);
+void tickit_window_set_geometry(TickitWindow *win, TickitRect rect);
+
+TickitPen *tickit_window_get_pen(const TickitWindow *win);
+void tickit_window_set_pen(TickitWindow *win, TickitPen *pen);
+void tickit_window_expose(TickitWindow *win, const TickitRect *exposed);
+void tickit_window_flush(TickitWindow *win);
+
+bool tickit_window_scrollrect(TickitWindow *win, const TickitRect *rect, int downward, int rightward, TickitPen *pen);
+bool tickit_window_scroll(TickitWindow *win, int downward, int rightward);
+bool tickit_window_scroll_with_children(TickitWindow *win, int downward, int rightward);
+
+void tickit_window_set_cursor_position(TickitWindow *win, int line, int col);
+void tickit_window_set_cursor_visible(TickitWindow *win, bool visible);
+void tickit_window_set_cursor_shape(TickitWindow *win, TickitCursorShape shape);
+
+void tickit_window_take_focus(TickitWindow *win);
+bool tickit_window_is_focused(const TickitWindow *win);
+void tickit_window_set_focus_child_notify(TickitWindow *win, bool notify);
+
+/*
+ * Debug support
+ */
+
+void tickit_debug_init(void);
+
+extern bool tickit_debug_enabled;
+
+void tickit_debug_logf(const char *flag, const char *fmt, ...);
+void tickit_debug_vlogf(const char *flag, const char *fmt, va_list args);
+
+void tickit_debug_set_func(void (*func)(const char *str, void *data), void *data);
+void tickit_debug_set_fh(FILE *fh);
+bool tickit_debug_open(const char *path);
 
 /*
  * Tickit events
@@ -393,17 +485,18 @@ typedef struct TickitWindow TickitWindow;
 
 /* bitmasks */
 enum TickitEventType {
-  TICKIT_EV_RESIZE     = 0x01, // Term = lines, cols
-  TICKIT_EV_KEY        = 0x02, // Term,Win = type(TickitKeyEventType), str
-  TICKIT_EV_MOUSE      = 0x04, // Term = type(TickitMouseEventType), button, line, col
-  TICKIT_EV_CHANGE     = 0x08, // Pen = {none}
-  TICKIT_EV_GEOMCHANGE = 0x10, // Win = rect
-  TICKIT_EV_EXPOSE     = 0x20, // Win = rect, rb
-  TICKIT_EV_FOCUS      = 0x40, // Win = type(TickitFocusEventType)
+  TICKIT_EV_RESIZE     = 0x01,
+  TICKIT_EV_KEY        = 0x02,
+  TICKIT_EV_MOUSE      = 0x04,
+  TICKIT_EV_CHANGE     = 0x08,
+  TICKIT_EV_GEOMCHANGE = 0x10,
+  TICKIT_EV_EXPOSE     = 0x20,
+  TICKIT_EV_FOCUS      = 0x40,
 
   TICKIT_EV_UNBIND = 0x80000000, // event handler is being unbound
 };
 
+// TICKIT_EV_RESIZE
 typedef struct {
   int lines, cols;
 } TickitResizeEventInfo;
@@ -413,6 +506,7 @@ typedef enum {
   TICKIT_KEYEV_TEXT,
 } TickitKeyEventType;
 
+// TICKIT_EV_KEY
 typedef struct {
   TickitKeyEventType type;
   int mod;
@@ -431,6 +525,7 @@ enum {
   TICKIT_MOUSEWHEEL_DOWN,
 };
 
+// TICKIT_EV_MOUSE
 typedef struct {
   TickitMouseEventType type;
   int button;
@@ -444,6 +539,29 @@ enum {
   TICKIT_MOD_CTRL  = 0x04,
 };
 
+// TICKIT_EV_GEOMCHANGE
+typedef struct {
+  TickitRect rect;
+  TickitRect oldrect;
+} TickitGeomchangeEventInfo;
+
+// TICKIT_EV_EXPOSE
+typedef struct {
+  TickitRect rect;
+  TickitRenderBuffer *rb;
+} TickitExposeEventInfo;
+
+typedef enum {
+  TICKIT_FOCUSEV_IN = 1,
+  TICKIT_FOCUSEV_OUT,
+} TickitFocusEventType;
+
+// TICKIT_EV_FOCUS
+typedef struct {
+  TickitFocusEventType type;
+  TickitWindow *win;
+} TickitFocusEventInfo;
+
 #endif
 
 #ifdef __cplusplus
