@@ -9,7 +9,7 @@ use strict;
 use warnings;
 
 BEGIN {
-   our $VERSION = '0.53';
+   our $VERSION = '0.54';
 }
 
 use Carp;
@@ -24,7 +24,6 @@ BEGIN {
    XSLoader::load( __PACKAGE__, our $VERSION );
 }
 
-use Tickit::Term qw( TERM_MOUSEMODE_DRAG );
 use Tickit::Window;
 
 use Tickit::Debug;
@@ -140,6 +139,8 @@ sub new
    unless( $term ) {
       my $writer = $self->_make_writer( $out );
 
+      require Tickit::Term;
+
       $term = Tickit::Term->find_for_term(
          writer        => $writer,
          input_handle  => $in,
@@ -422,7 +423,7 @@ sub setup_term
 
    $term->setctl_int( altscreen => 1 ) if $self->{use_altscreen};
    $term->setctl_int( cursorvis => 0 );
-   $term->setctl_int( mouse     => TERM_MOUSEMODE_DRAG );
+   $term->setctl_int( mouse     => Tickit::Term::TERM_MOUSEMODE_DRAG );
    $term->clear;
 
    if( my $widget = $self->{root_widget} ) {
