@@ -1,26 +1,18 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2014-2015 -- leonerd@leonerd.org.uk
 
 package Tickit::Debug;
 
 use strict;
 use warnings;
-use 5.010; # //
 
-# Behaviour depends on some environment variables
-use constant DEBUG_FLAGS => { map { +$_ => 1 } split m/,/, $ENV{TICKIT_DEBUG_FLAGS} // "" };
-use constant DEBUG => scalar keys %{+DEBUG_FLAGS};
+use constant DEBUG => _enabled();
 
 use Exporter 'import';
 our @EXPORT = qw( DEBUG );
 
-use Time::HiRes qw( gettimeofday );
-use POSIX qw( strftime );
-
-our $debug_fh;
-
 =head1 NAME
 
 C<Tickit::Debug> - debug logging support for C<Tickit>
@@ -138,26 +130,6 @@ PID of the process, named as
 
 =cut
 
-if( DEBUG ) {
-   if( $ENV{TICKIT_DEBUG_FD} ) {
-      require IO::Handle;
-
-      $debug_fh = IO::Handle->new;
-      $debug_fh->fdopen( $ENV{TICKIT_DEBUG_FD}, "w" ) or
-         undef $debug_fh;
-   }
-   else {
-      open $debug_fh, ">", $ENV{TICKIT_DEBUG_FILE} // "tickit-$$.log"
-         or warn("Unable to open Tickit debug logging file - $!; logging disabled\n");
-   }
-
-   if( $debug_fh ) {
-      $debug_fh->autoflush(1);
-
-      $debug_fh->printf( "Logging started\n" );
-   }
-}
-
 =head1 METHODS
 
 =cut
@@ -181,21 +153,11 @@ sub log :method
    shift;
    my ( $flag, $format, @args ) = @_;
 
-   return unless $debug_fh;
-
-   $flag =~ m/^([A-Z]+)/;
-   my $flag_category = $1;
-
-   return unless DEBUG_FLAGS->{$flag} ||
-      ( $flag_category && DEBUG_FLAGS->{$flag_category} ) ||
-      DEBUG_FLAGS->{"*"};
+   return unless _enabled();
 
    my $message = sprintf $format, map { ref eq "CODE" ? $_->() : $_ } @args;
 
-   my ( $sec, $usec ) = gettimeofday;
-   my $timestamp = strftime( "%H:%M:%S", localtime $sec );
-
-   $debug_fh->printf( "%s.%03d [%-3s]: %s\n", $timestamp, $usec / 1000, $flag, $message );
+   _log( $flag, $message );
 }
 
 =head1 AUTHOR
