@@ -7,10 +7,8 @@ package Tickit::RenderBuffer;
 
 use strict;
 use warnings;
-use feature qw( switch );
-no if $] >= 5.017011, warnings => 'experimental::smartmatch';
 
-our $VERSION = '0.53';
+our $VERSION = '0.54';
 
 use Carp;
 use Scalar::Util qw( refaddr );
@@ -588,174 +586,6 @@ buffer will be cleared and reset back to initial state.
 
 =cut
 
-## Tickit::Debug wrapping support
-use Tickit::Debug;
-
-if( Tickit::Debug::DEBUG ) {
-   local *around = sub {
-      my ( $method, $code ) = @_;
-
-      my $orig = __PACKAGE__->can( $method ) or croak __PACKAGE__." cannot ->$method";
-
-      no strict 'refs';
-      no warnings 'redefine';
-      *$method = sub { $code->( $orig, @_ ) };
-   };
-
-   my $INDENT = "";
-   *lg = sub {
-      Tickit::Debug->log( $_[0], "${INDENT}$_[1]", @_[2..$#_] );
-   };
-
-   # Bs == stack
-   around( save => sub {
-      my $orig = shift;
-      lg( Bs => "+-Save" );
-      $INDENT = "|  $INDENT";
-      $orig->( @_ );
-   });
-   around( savepen => sub {
-      my $orig = shift;
-      lg( Bs => "+-Savepen" );
-      $INDENT = "|  $INDENT";
-      $orig->( @_ );
-   });
-   around( restore => sub {
-      my $orig = shift;
-      $INDENT =~ s/^\|  //;
-      lg( Bs => "+-Restore" );
-      $orig->( @_ );
-   });
-   around( reset => sub {
-      my $orig = shift;
-      $INDENT = "";
-      $orig->( @_ );
-   });
-
-   # Bd == drawing operations
-   around( clear => sub {
-      my $orig = shift;
-      lg( Bd => "Clear" );
-      $orig->( @_ );
-   });
-
-   around( skip => sub {
-      my $orig = shift;
-      my ( $rb, $len ) = @_;
-      lg( Bd => "Skip (%d..%d,%d) +%d", sub { $rb->col, $rb->col + $len, $rb->line, $len } );
-      $orig->( @_ );
-   });
-   around( skip_at => sub {
-      my $orig = shift;
-      my ( $rb, $line, $col, $len ) = @_;
-      lg( Bd => "Skip (%d..%d,%d)", sub { $line, $col, $col + $len, $line } );
-      $orig->( @_ );
-   });
-   around( skip_to => sub {
-      my $orig = shift;
-      my ( $rb, $col ) = @_;
-      lg( Bd => "Skip (%d..%d,%d) +%d", sub { $rb->col, $col, $rb->line, $col - $rb->col } ) if $col > $rb->col;
-      $orig->( @_ );
-   });
-
-   around( erase => sub {
-      my $orig = shift;
-      my ( $rb, $len ) = @_;
-      lg( Bd => "Erase (%d..%d,%d) +%d", sub { $rb->col, $rb->col + $len, $rb->line, $len } );
-      $orig->( @_ );
-   });
-   around( erase_at => sub {
-      my $orig = shift;
-      my ( $rb, $line, $col, $len ) = @_;
-      lg( Bd => "Erase (%d..%d,%d)", sub { $col, $col + $len, $line } );
-      $orig->( @_ );
-   });
-   around( erase_to => sub {
-      my $orig = shift;
-      my ( $rb, $col ) = @_;
-      lg( Bd => "Erase (%d..%d,%d) +%d", sub { $rb->col, $col, $rb->line, $col - $rb->col } ) if $col > $rb->col;
-      $orig->( @_ );
-   });
-   around( eraserect => sub {
-      my $orig = shift;
-      my ( $rb, $rect ) = @_;
-      lg( Bd => "Erase (%d,%d)..(%d,%d)", sub { $rect->left, $rect->top, $rect->right, $rect->bottom } );
-      $orig->( @_ );
-   });
-
-   around( text => sub {
-      my $orig = shift;
-      my ( $rb ) = @_;
-      my $col = $rb->col;
-      my $len = $orig->( @_ );
-      lg( Bd => "Text (%d..%d,%d) +%d", sub { $col, $col + $len, $rb->line, $len } );
-      return $len;
-   });
-   around( text_at => sub {
-      my $orig = shift;
-      my ( $rb, $line, $col ) = @_;
-      my $len = $orig->( @_ );
-      lg( Bd => "Text (%d..%d,%d)", sub { $col, $col + $len, $line } );
-      return $len;
-   });
-
-   around( char => sub {
-      my $orig = shift;
-      my ( $rb ) = @_;
-      lg( Bd => "Char (%d..%d,%d) +%d", sub { $rb->col, $rb->col + 1, $rb->line, 1 } );
-      $orig->( @_ );
-   });
-   around( char_at => sub {
-      my $orig = shift;
-      my ( $rb, $line, $col ) = @_;
-      lg( Bd => "Char (%d..%d,%d)", sub { $col, $col + 1, $line } );
-      $orig->( @_ );
-   });
-
-   around( hline_at => sub {
-      my $orig = shift;
-      my ( $rb, $line, $startcol, $endcol ) = @_;
-      lg( Bd => "HLine (%d..%d,%d)", $startcol, $endcol, $line );
-      $orig->( @_ );
-   });
-   around( vline_at => sub {
-      my $orig = shift;
-      my ( $rb, $startline, $endline, $col ) = @_;
-      lg( Bd => "VLine (%d,%d..%d)", $col, $startline, $endline );
-      $orig->( @_ );
-   });
-
-   # Bf == flush
-   around( flush_to_term => sub {
-      my $orig = shift;
-      $INDENT = "";
-      lg( Bf => "Flush" );
-      $orig->( @_ );
-   });
-
-   # Bt == transformations (translate, clip, mask)
-   around( translate => sub {
-      my $orig = shift;
-      my ( $rb, $downward, $rightward ) = @_;
-      lg( Bt => "Translate (+%d,+%d)", $rightward, $downward );
-      $orig->( @_ );
-   });
-   around( clip => sub {
-      my $orig = shift;
-      my ( $rb, $rect ) = @_;
-      lg( Bd => "Clip (%d,%d)..(%d,%d)", sub { $rect->left, $rect->top, $rect->right, $rect->bottom } );
-      $orig->( @_ );
-   });
-   around( mask => sub {
-      my $orig = shift;
-      my ( $rb, $rect ) = @_;
-      lg( Bd => "Mask (%d,%d)..(%d,%d)", sub { $rect->left, $rect->top, $rect->right, $rect->bottom } );
-      $orig->( @_ );
-   });
-}
-
-undef *around; # delete it as a method
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
