@@ -1,23 +1,34 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2015 -- leonerd@leonerd.org.uk
 
 package Tickit::Term;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.53';
+our $VERSION = '0.54';
 
 use Carp;
 
 # Load the XS code
-require Tickit;
+use Tickit;
 
 # We export some constants
 use Exporter 'import';
 
+# Old names for these
+use constant {
+   TERM_CURSORSHAPE_BLOCK    => CURSORSHAPE_BLOCK,
+   TERM_CURSORSHAPE_UNDER    => CURSORSHAPE_UNDER,
+   TERM_CURSORSHAPE_LEFT_BAR => CURSORSHAPE_LEFT_BAR,
+};
+
+push our @EXPORT_OK, qw(
+   TERM_CURSORSHAPE_BLOCK TERM_CURSORSHAPE_UNDER TERM_CURSORSHAPE_LEFT_BAR
+);
+
 =head1 NAME
 
 C<Tickit::Term> - terminal formatting abstraction
@@ -380,8 +391,7 @@ Enables cursor blinking mode
 =item TERMCTL_CURSORSHAPE
 
 Sets the shape of the cursor. C<$value> should be one of
-C<TERM_CURSORSHAPE_BLOCK>, C<TERM_CURSORSHAPE_UNDER> or
-C<TERM_CURSORSHAPE_LEFT_BAR>.
+C<CURSORSHAPE_BLOCK>, C<CURSORSHAPE_UNDER> or C<CURSORSHAPE_LEFT_BAR>.
 
 =item TERMCTL_KEYPAD_APP
 
