@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use 5.010; # //
 
-our $VERSION = '0.53';
+our $VERSION = '0.54';
 
 use Carp;
 
@@ -227,7 +227,6 @@ sub _close
 
    defined and $_->_close for @{ $self->{child_windows} };
    undef $self->{child_windows};
-   @{ $self->{pending_geom_changes} } = ();
 }
 
 sub DESTROY
@@ -245,7 +244,10 @@ sub close
 {
    my $self = shift;
    $self->_close;
-   $self->{parent}->_change_children( remove => $self ) if $self->{parent};
+   if( my $parent = $self->{parent} ) {
+      $parent->_change_children( remove => $self );
+      $parent->expose( $self->rect ) if $self->{visible};
+   }
 }
 
 sub _do_change_children
@@ -290,6 +292,21 @@ sub _change_children
    $root->_needs_later;
 }
 
+sub _make_sub
+{
+   my $self = shift;
+   my ( $top, $left, $lines, $cols ) = @_;
+
+   my $sub = bless {
+      parent => $self,
+   }, ref $self;
+   $sub->_init;
+
+   $sub->change_geometry( $top, $left, $lines, $cols );
+
+   return $sub;
+}
+
 =head2 $sub = $win->make_sub( $top, $left, $lines, $cols )
 
 Constructs a new sub-window of the given geometry, and places it at the end of
@@ -300,17 +317,12 @@ the child window list; below any other siblings.
 sub make_sub
 {
    my $self = shift;
-   my ( $top, $left, $lines, $cols ) = @_;
-
-   my $sub = bless {
-      parent  => $self,
-   }, ref $self;
-   $sub->_init;
-
    $self->_reap_dead_children;
 
-   $sub->change_geometry( $top, $left, $lines, $cols );
-   $self->_change_children( insert => $sub, -1 );
+   my $sub = $self->_make_sub( @_ );
+
+   $self->_do_change_children( insert => $sub, -1 );
+   $self->expose( $sub->rect );
 
    return $sub;
 }
@@ -325,10 +337,13 @@ hidden. This avoids having to call C<hide> separately afterwards.
 sub make_hidden_sub
 {
    my $self = shift;
+   $self->_reap_dead_children;
 
-   my $sub = $self->make_sub( @_ );
+   my $sub = $self->_make_sub( @_ );
    $sub->{visible} = 0;
 
+   $self->_do_change_children( insert => $sub, -1 );
+
    return $sub;
 }
 
@@ -342,17 +357,12 @@ of the child window list; above any other siblings.
 sub make_float
 {
    my $self = shift;
-   my ( $top, $left, $lines, $cols ) = @_;
-
-   my $sub = bless {
-      parent  => $self,
-   }, ref $self;
-   $sub->_init;
-
    $self->_reap_dead_children;
 
-   $sub->change_geometry( $top, $left, $lines, $cols );
-   $self->_change_children( insert => $sub, 0 );
+   my $sub = $self->_make_sub( @_ );
+
+   $self->_do_change_children( insert => $sub, 0 );
+   $self->expose( $sub->rect );
 
    return $sub;
 }
@@ -1405,14 +1415,20 @@ sub _scrollrectset
       my $damageset = $win->{damage};
       my @damage = $damageset->rects;
       $damageset->clear;
+
       foreach my $r ( @damage ) {
          $damageset->add( $r ), next if $r->bottom < $rect->top or $r->top > $rect->bottom or
                                         $r->right < $rect->left or $r->left > $rect->right;
-         my $inside = $r->intersect( $rect );
-         my @outside = $r->subtract( $rect );
 
+         my @outside = $r->subtract( $rect );
          $damageset->add( $_ ) for @outside;
-         $damageset->add( $inside->translate( -$downward, -$rightward ) ) if $inside;
+
+         my $inside = $r->intersect( $rect );
+         if( $inside ) {
+            $inside = $inside->translate( -$downward, -$rightward )
+                             ->intersect( $rect );
+            $damageset->add( $inside ) if $inside;
+         }
       }
 
       Tickit::Debug->log( Wsr => "Term scrollrect %s by %+d,%+d",
@@ -1582,7 +1598,7 @@ sub cursor_visible
 Sets the shape that the terminal cursor will have if this window has focus.
 This method does I<not> force the window to take the focus though; for that
 see C<take_focus>. Valid values for C<$shape> are the various
-C<TERM_CURSORSHAPE_*> constants from L<Tickit::Term>.
+C<CURSORSHAPE_*> constants from L<Tickit::Term>.
 
 =cut
 
@@ -1701,7 +1717,7 @@ sub restore
 
    if( $win and $win->is_focused and $win->{cursor_visible} and
        ( $win->_get_span_visibility( $win->{cursor_line}, $win->{cursor_col} ) )[0] ) {
-      my $cursorshape = $win->{cursor_shape} // Tickit::Term::TERM_CURSORSHAPE_BLOCK;
+      my $cursorshape = $win->{cursor_shape} // Tickit::Term::CURSORSHAPE_BLOCK;
       $term->setctl_int( cursorvis => 1 );
       $win->term->goto( $win->{cursor_line} + $win->abs_top, $win->{cursor_col} + $win->abs_left );
       $win->term->setctl_int( cursorshape => $cursorshape );
