@@ -1,15 +1,10 @@
-#include "tickit-window.h"
+#include "tickit.h"
 #include "hooklists.h"
 
 #include <stdio.h>
 
-/* TODO:
- *   Scrolling
- *   Input events (mouse, keyboard)
- */
-
-#define ROOT_AS_WINDOW(root) (TickitWindow*)root
-#define WINDOW_AS_ROOT(win) (TickitRootWindow*)win
+#define ROOT_AS_WINDOW(root) ((TickitWindow*)root)
+#define WINDOW_AS_ROOT(win)  ((TickitRootWindow*)win)
 
 typedef enum {
   TICKIT_HIERARCHY_INSERT_FIRST,
@@ -31,7 +26,7 @@ struct TickitWindow {
   struct {
     int line;
     int col;
-    TickitTermCursorShape shape;
+    TickitCursorShape shape;
     bool visible;
   } cursor;
   bool is_visible;
@@ -74,38 +69,59 @@ static void _purge_hierarchy_changes(TickitWindow *win);
 static int _handle_key(TickitWindow *win, TickitKeyEventInfo *args);
 static int _handle_mouse(TickitWindow *win, TickitMouseEventInfo *args);
 
-static int on_term(TickitTerm *term, TickitEventType ev, void *_info, void *data)
+static int on_term(TickitTerm *term, TickitEventType ev, void *_info, void *user)
 {
-  TickitRootWindow *root = data;
+  TickitRootWindow *root = user;
+  TickitWindow *win = ROOT_AS_WINDOW(root);
 
   if(ev & TICKIT_EV_RESIZE) {
     TickitResizeEventInfo *info = _info;
-    tickit_window_resize(ROOT_AS_WINDOW(root), info->lines, info->cols);
+    int oldlines = win->rect.lines;
+    int oldcols  = win->rect.cols;
+
+    tickit_window_resize(win, info->lines, info->cols);
+
+    if(info->lines > oldlines) {
+      TickitRect damage = {
+        .top   = oldlines,
+        .left  = 0,
+        .lines = info->lines - oldlines,
+        .cols  = info->cols,
+      };
+      tickit_window_expose(win, &damage);
+    }
+
+    if(info->cols > oldcols) {
+      TickitRect damage = {
+        .top   = 0,
+        .left  = oldcols,
+        .lines = oldlines,
+        .cols  = info->cols - oldcols,
+      };
+      tickit_window_expose(win, &damage);
+    }
   }
 
   if(ev & TICKIT_EV_KEY)
-    _handle_key(ROOT_AS_WINDOW(root), (TickitKeyEventInfo *)_info);
+    _handle_key(win, (TickitKeyEventInfo *)_info);
 
   if(ev & TICKIT_EV_MOUSE)
-    _handle_mouse(ROOT_AS_WINDOW(root), (TickitMouseEventInfo *)_info);
+    _handle_mouse(win, (TickitMouseEventInfo *)_info);
 
   return 1;
 }
 
-static void init_window(TickitWindow *win, TickitWindow *parent, int top, int left, int lines, int cols)
+static void init_window(TickitWindow *win, TickitWindow *parent, TickitRect rect)
 {
   win->parent = parent;
   win->first_child = NULL;
   win->next = NULL;
   win->focused_child = NULL;
-  win->pen = NULL;
-  win->rect.top = top;
-  win->rect.left = left;
-  win->rect.lines = lines;
-  win->rect.cols = cols;
+  win->pen = tickit_pen_new();
+  win->rect = rect;
   win->cursor.line = 0;
   win->cursor.col = 0;
-  win->cursor.shape = TICKIT_TERM_CURSORSHAPE_BLOCK;
+  win->cursor.shape = TICKIT_CURSORSHAPE_BLOCK;
   win->cursor.visible = true;
   win->is_visible = true;
   win->is_focused = false;
@@ -115,17 +131,6 @@ static void init_window(TickitWindow *win, TickitWindow *parent, int top, int le
   win->hooks = NULL;
 }
 
-static TickitWindow* new_window(TickitWindow *parent, int top, int left, int lines, int cols)
-{
-  TickitWindow *win = malloc(sizeof(TickitWindow));
-  if(!win)
-    return NULL;
-
-  init_window(win, parent, top, left, lines, cols);
-
-  return win;
-}
-
 TickitWindow* tickit_window_new_root(TickitTerm *term)
 {
   int lines, cols;
@@ -135,7 +140,7 @@ TickitWindow* tickit_window_new_root(TickitTerm *term)
   if(!root)
     return NULL;
 
-  init_window(ROOT_AS_WINDOW(root), NULL, 0, 0, lines, cols);
+  init_window(ROOT_AS_WINDOW(root), NULL, (TickitRect) { .top = 0, .left = 0, .lines = lines, .cols = cols });
 
   root->term = term;
   root->hierarchy_changes = NULL;
@@ -152,6 +157,8 @@ TickitWindow* tickit_window_new_root(TickitTerm *term)
   root->event_id = tickit_term_bind_event(term, TICKIT_EV_RESIZE|TICKIT_EV_KEY|TICKIT_EV_MOUSE,
       &on_term, root);
 
+  tickit_window_expose(ROOT_AS_WINDOW(root), NULL);
+
   return ROOT_AS_WINDOW(root);
 }
 
@@ -164,39 +171,31 @@ static TickitRootWindow *_get_root(const TickitWindow *win)
   return WINDOW_AS_ROOT(root);
 }
 
-TickitWindow *tickit_window_new_subwindow(TickitWindow *parent, int top, int left, int lines, int cols)
+TickitWindow *tickit_window_new(TickitWindow *parent, TickitRect rect, TickitWindowFlags flags)
 {
-  TickitWindow *win = new_window(parent, top, left, lines, cols);
-  _request_hierarchy_change(TICKIT_HIERARCHY_INSERT_LAST, win);
-  return win;
-}
+  if(flags & TICKIT_WINDOW_ROOT_PARENT)
+    while(parent->parent) {
+      rect.top  += parent->rect.top;
+      rect.left += parent->rect.left;
+      parent = parent->parent;
+    }
 
-TickitWindow *tickit_window_new_hidden_subwindow(TickitWindow *parent, int top, int left, int lines, int cols)
-{
-  TickitWindow *win = new_window(parent, top, left, lines, cols);
-  _request_hierarchy_change(TICKIT_HIERARCHY_INSERT_LAST, win);
-  win->is_visible = false;
-  return win;
-}
+  TickitWindow *win = malloc(sizeof(TickitWindow));
+  if(!win)
+    return NULL;
 
-TickitWindow *tickit_window_new_float(TickitWindow *parent, int top, int left, int lines, int cols)
-{
-  TickitWindow *win = new_window(parent, top, left, lines, cols);
-  _request_hierarchy_change(TICKIT_HIERARCHY_INSERT_FIRST, win);
-  return win;
-}
+  init_window(win, parent, rect);
+
+  if(flags & TICKIT_WINDOW_HIDDEN)
+    win->is_visible = false;
+  if(flags & TICKIT_WINDOW_STEAL_INPUT)
+    win->steal_input = true;
+
+  _do_hierarchy_change(
+    (flags & TICKIT_WINDOW_LOWEST) ? TICKIT_HIERARCHY_INSERT_LAST : TICKIT_HIERARCHY_INSERT_FIRST,
+    parent, win
+  );
 
-TickitWindow *tickit_window_new_popup(TickitWindow *parent, int top, int left, int lines, int cols)
-{
-  TickitWindow *root = parent;
-  while(root->parent) {
-    top += root->rect.top;
-    left += root->rect.left;
-    root = root->parent;
-  }
-  TickitWindow *win = new_window(root, top, left, lines, cols);
-  _request_hierarchy_change(TICKIT_HIERARCHY_INSERT_FIRST, win);
-  win->steal_input = true;
   return win;
 }
 
@@ -214,7 +213,8 @@ void tickit_window_destroy(TickitWindow *win)
 {
   tickit_hooklist_unbind_and_destroy(win->hooks, win);
 
-  win->pen = NULL;
+  if(win->pen)
+    tickit_pen_unref(win->pen);
 
   for(TickitWindow *child = win->first_child; child; child = child->next)
     tickit_window_destroy(child);
@@ -287,46 +287,19 @@ bool tickit_window_is_visible(TickitWindow *win)
   return win->is_visible;
 }
 
-int tickit_window_top(const TickitWindow *win)
+TickitRect tickit_window_get_geometry(const TickitWindow *win)
 {
-  return win->rect.top;
+  return win->rect;
 }
 
-int tickit_window_abs_top(const TickitWindow *win)
+TickitRect tickit_window_get_abs_geometry(const TickitWindow *win)
 {
-  int top = win->rect.top;
-  TickitWindow* parent = win->parent;
-  while(parent) {
-    top += parent->rect.top;
-    parent = parent->parent;
-  }
-  return top;
-}
+  TickitRect geom = win->rect;
 
-int tickit_window_left(const TickitWindow *win)
-{
-  return win->rect.left;
-}
+  for(win = win->parent; win; win = win->parent)
+    tickit_rect_translate(&geom, win->rect.top, win->rect.left);
 
-int tickit_window_abs_left(const TickitWindow *win)
-{
-  int left = win->rect.left;
-  TickitWindow* parent = win->parent;
-  while(parent) {
-    left += parent->rect.left;
-    parent = parent->parent;
-  }
-  return left;
-}
-
-int tickit_window_lines(const TickitWindow *win)
-{
-  return win->rect.lines;
-}
-
-int tickit_window_cols(const TickitWindow *win)
-{
-  return win->rect.cols;
+  return geom;
 }
 
 int tickit_window_bottom(const TickitWindow *win)
@@ -341,40 +314,59 @@ int tickit_window_right(const TickitWindow *win)
 
 void tickit_window_resize(TickitWindow *win, int lines, int cols)
 {
-  tickit_window_set_geometry(win, win->rect.top, win->rect.left, lines, cols);
+  tickit_window_set_geometry(win, (TickitRect){
+      .top   = win->rect.top,
+      .left  = win->rect.left,
+      .lines = lines,
+      .cols  = cols}
+  );
 }
 
 void tickit_window_reposition(TickitWindow *win, int top, int left)
 {
-  tickit_window_set_geometry(win, top, left, win->rect.lines, win->rect.cols);
+  tickit_window_set_geometry(win, (TickitRect){
+      .top   = top,
+      .left  = left,
+      .lines = win->rect.lines,
+      .cols  = win->rect.cols}
+  );
 
   if(win->is_focused)
     _request_restore(_get_root(win));
 }
 
-void tickit_window_set_geometry(TickitWindow *win, int top, int left, int lines, int cols)
+void tickit_window_set_geometry(TickitWindow *win, TickitRect geom)
 {
-  if((win->rect.top != top) ||
-     (win->rect.left != left) ||
-     (win->rect.lines != lines) ||
-     (win->rect.cols != cols))
+  if((win->rect.top != geom.top) ||
+     (win->rect.left != geom.left) ||
+     (win->rect.lines != geom.lines) ||
+     (win->rect.cols != geom.cols))
   {
-    win->rect.top = top;
-    win->rect.left = left;
-    win->rect.lines = lines;
-    win->rect.cols = cols;
-
     TickitGeomchangeEventInfo info = {
-      .rect = { .top = top, .left = left, .lines = lines, .cols = cols },
+      .rect = geom,
+      .oldrect = win->rect,
     };
+
+    win->rect = geom;
+
     run_events(win, TICKIT_EV_GEOMCHANGE, &info);
   }
 }
 
+TickitPen *tickit_window_get_pen(const TickitWindow *win)
+{
+  return win->pen;
+}
+
 void tickit_window_set_pen(TickitWindow *win, TickitPen *pen)
 {
-  /* TODO: Refcounting the pen would be nice. Until then, we assume we don't own it. */
-  win->pen = pen;
+  if(win->pen)
+    tickit_pen_unref(win->pen);
+
+  if(pen)
+    win->pen = tickit_pen_ref(pen);
+  else
+    win->pen = NULL;
 }
 
 void tickit_window_expose(TickitWindow *win, const TickitRect *exposed)
@@ -451,6 +443,38 @@ static void _request_later_processing(TickitRootWindow *root)
   root->needs_later_processing = true;
 }
 
+static bool _cell_visible(TickitWindow *win, int line, int col)
+{
+  TickitWindow *prev;
+  while(win) {
+    if(line < 0 || line >= win->rect.lines ||
+       col  < 0 || col  >= win->rect.cols)
+      return false;
+
+    for(TickitWindow *child = win->first_child; child; child = child->next) {
+      if(prev && child == prev)
+        break;
+      if(!child->is_visible)
+        continue;
+
+      if(line < child->rect.top  || line >= child->rect.top + child->rect.lines)
+        continue;
+      if(col  < child->rect.left || col  >= child->rect.left + child->rect.cols)
+        continue;
+
+      return false;
+    }
+
+    line += win->rect.top;
+    col  += win->rect.left;
+
+    prev = win;
+    win = win->parent;
+  }
+
+  return true;
+}
+
 static void _do_restore(TickitRootWindow *root)
 {
   TickitWindow *win = ROOT_AS_WINDOW(root);
@@ -465,11 +489,12 @@ static void _do_restore(TickitRootWindow *root)
     win = win->focused_child;
   }
 
-  if(win && win->is_focused && win->cursor.visible) {
-    /* TODO finish the visibility check here. */
+  if(win && win->is_focused && win->cursor.visible &&
+     _cell_visible(win, win->cursor.line, win->cursor.col)) {
     tickit_term_setctl_int(root->term, TICKIT_TERMCTL_CURSORVIS, 1);
-    int cursor_line = win->cursor.line + tickit_window_abs_top(win);
-    int cursor_col = win->cursor.col + tickit_window_abs_left(win);
+    TickitRect abs_geom = tickit_window_get_abs_geometry(win);
+    int cursor_line = win->cursor.line + abs_geom.top;
+    int cursor_col = win->cursor.col + abs_geom.left;
     tickit_term_goto(root->term, cursor_line, cursor_col);
     tickit_term_setctl_int(root->term, TICKIT_TERMCTL_CURSORSHAPE, win->cursor.shape);
   }
@@ -479,10 +504,10 @@ static void _do_restore(TickitRootWindow *root)
   tickit_term_flush(root->term);
 }
 
-void tickit_window_tick(TickitWindow *win)
+void tickit_window_flush(TickitWindow *win)
 {
   if(win->parent)
-    // Can't tick non-root.
+    // Can't flush non-root.
     return;
 
   TickitRootWindow *root = WINDOW_AS_ROOT(win);
@@ -614,13 +639,11 @@ static void _do_hierarchy_change(HierarchyChangeType change, TickitWindow *paren
     case TICKIT_HIERARCHY_INSERT_LAST:
       _do_hierarchy_insert_last(parent, win);
       break;
-    case TICKIT_HIERARCHY_REMOVE: {
-        _do_hierarchy_remove(parent, win);
-        if(parent->focused_child && parent->focused_child == win) {
-          parent->focused_child = NULL;
-        }
-        break;
-      }
+    case TICKIT_HIERARCHY_REMOVE:
+      _do_hierarchy_remove(parent, win);
+      if(parent->focused_child && parent->focused_child == win)
+        parent->focused_child = NULL;
+      break;
     case TICKIT_HIERARCHY_RAISE:
       _do_hierarchy_raise(parent, win);
       break;
@@ -680,7 +703,187 @@ static void _purge_hierarchy_changes(TickitWindow *win)
   }
 }
 
-void tickit_window_cursor_at(TickitWindow *win, int line, int col)
+static bool _scrollrectset(TickitWindow *win, TickitRectSet *visible, int downward, int rightward, TickitPen *pen)
+{
+  TickitWindow *origwin = win;
+
+  int abs_top = 0, abs_left = 0;
+
+  while(win) {
+    if(!win->is_visible)
+      return false;
+
+    tickit_pen_copy(pen, win->pen, false);
+
+    TickitWindow *parent = win->parent;
+    if(!parent)
+      break;
+
+    abs_top  += win->rect.top;
+    abs_left += win->rect.left;
+    tickit_rectset_translate(visible, win->rect.top, win->rect.left);
+
+    for(TickitWindow *sib = parent->first_child; sib; sib = sib->next) {
+      if(sib == win)
+        break;
+      if(!sib->is_visible)
+        continue;
+
+      tickit_rectset_subtract(visible, &sib->rect);
+    }
+
+    win = parent;
+  }
+
+  TickitTerm *term = WINDOW_AS_ROOT(win)->term;
+
+  int n = tickit_rectset_rects(visible);
+  TickitRect rects[n];
+  tickit_rectset_get_rects(visible, rects, n);
+
+  bool ret = true;
+  bool done_pen = false;
+
+  for(int i = 0; i < n; i++) {
+    TickitRect rect = rects[i];
+
+    TickitRect origrect = rect;
+    tickit_rect_translate(&origrect, -abs_top, -abs_left);
+
+    if(abs(downward) >= rect.lines || abs(rightward) >= rect.cols) {
+      tickit_window_expose(origwin, &origrect);
+      continue;
+    }
+
+    // TODO: This may be more efficiently done with some rectset operations
+    //   instead of completely resetting and rebuilding the set
+    TickitRectSet *damageset = WINDOW_AS_ROOT(win)->damage;
+    int n_damage = tickit_rectset_rects(damageset);
+    TickitRect damage[n_damage];
+    tickit_rectset_get_rects(damageset, damage, n_damage);
+    tickit_rectset_clear(damageset);
+
+    for(int j = 0; j < n_damage; j++) {
+      TickitRect r = damage[j];
+
+      if(tickit_rect_bottom(&r) < rect.top || r.top > tickit_rect_bottom(&rect) ||
+         tickit_rect_right(&r) < rect.left || r.left > tickit_rect_right(&rect)) {
+        tickit_rectset_add(damageset, &r);
+        continue;
+      }
+
+      TickitRect outside[4];
+      int n_outside;
+      if((n_outside = tickit_rect_subtract(outside, &r, &rect))) {
+        for(int k = 0; k < n_outside; k++)
+          tickit_rectset_add(damageset, outside+k);
+      }
+
+      TickitRect inside;
+      if(tickit_rect_intersect(&inside, &r, &rect)) {
+        tickit_rect_translate(&inside, -downward, -rightward);
+        if(tickit_rect_intersect(&inside, &inside, &rect))
+          tickit_rectset_add(damageset, &inside);
+      }
+    }
+
+    if(!done_pen) {
+      // TODO: only bg matters
+      tickit_term_setpen(term, pen);
+      done_pen = true;
+    }
+
+    if(tickit_term_scrollrect(term, rect, downward, rightward)) {
+      if(downward > 0) {
+        // "scroll down" means lines moved upward, so the bottom needs redrawing
+        tickit_window_expose(origwin, &(TickitRect){
+            .top  = origrect.top + origrect.lines - downward, .lines = downward,
+            .left = origrect.left,                            .cols  = rect.cols
+        });
+      }
+      else if(downward < 0) {
+        // "scroll up" means lines moved downward, so top needs redrawing
+        tickit_window_expose(origwin, &(TickitRect){
+            .top  = origrect.top,  .lines = -downward,
+            .left = origrect.left, .cols  = rect.cols,
+        });
+      }
+
+      if(rightward > 0) {
+        // "scroll right" means columns moved leftward, so the right edge needs redrawing
+        tickit_window_expose(origwin, &(TickitRect){
+            .top  = origrect.top,                              .lines = rect.lines,
+            .left = origrect.left + origrect.cols - rightward, .cols  = rightward,
+        });
+      }
+      else if(rightward < 0) {
+        tickit_window_expose(origwin, &(TickitRect){
+            .top  = origrect.top,  .lines = rect.lines,
+            .left = origrect.left, .cols  = -rightward,
+        });
+      }
+    }
+    else {
+      tickit_window_expose(origwin, &origrect);
+      ret = false;
+    }
+  }
+
+  return ret;
+}
+
+static bool _scroll(TickitWindow *win, const TickitRect *origrect, int downward, int rightward, TickitPen *pen, bool mask_children)
+{
+  TickitRect rect;
+  TickitRect selfrect = { .top = 0, .left = 0, .lines = win->rect.lines, .cols = win->rect.cols };
+
+  if(!tickit_rect_intersect(&rect, &selfrect, origrect))
+    return false;
+
+  if(pen)
+    pen = tickit_pen_ref(pen);
+  else
+    pen = tickit_pen_new();
+
+  TickitRectSet *visible = tickit_rectset_new();
+
+  tickit_rectset_add(visible, &rect);
+
+  if(mask_children)
+    for(TickitWindow *child = win->first_child; child; child = child->next) {
+      if(!child->is_visible)
+        continue;
+      tickit_rectset_subtract(visible, &child->rect);
+    }
+
+  bool ret = _scrollrectset(win, visible, downward, rightward, pen);
+
+  tickit_rectset_destroy(visible);
+  tickit_pen_unref(pen);
+
+  return ret;
+}
+
+bool tickit_window_scrollrect(TickitWindow *win, const TickitRect *rect, int downward, int rightward, TickitPen *pen)
+{
+  return _scroll(win, rect, downward, rightward, pen, true);
+}
+
+bool tickit_window_scroll(TickitWindow *win, int downward, int rightward)
+{
+  return _scroll(win,
+    &((TickitRect){ .top = 0, .left = 0, .lines = win->rect.lines, .cols = win->rect.cols }),
+    downward, rightward, NULL, true);
+}
+
+bool tickit_window_scroll_with_children(TickitWindow *win, int downward, int rightward)
+{
+  return _scroll(win,
+    &((TickitRect){ .top = 0, .left = 0, .lines = win->rect.lines, .cols = win->rect.cols }),
+    downward, rightward, NULL, false);
+}
+
+void tickit_window_set_cursor_position(TickitWindow *win, int line, int col)
 {
   win->cursor.line = line;
   win->cursor.col = col;
@@ -689,7 +892,7 @@ void tickit_window_cursor_at(TickitWindow *win, int line, int col)
     _request_restore(_get_root(win));
 }
 
-void tickit_window_cursor_visible(TickitWindow *win, bool visible)
+void tickit_window_set_cursor_visible(TickitWindow *win, bool visible)
 {
   win->cursor.visible = visible;
 
@@ -697,7 +900,7 @@ void tickit_window_cursor_visible(TickitWindow *win, bool visible)
     _request_restore(_get_root(win));
 }
 
-void tickit_window_cursor_shape(TickitWindow *win, TickitTermCursorShape shape)
+void tickit_window_set_cursor_shape(TickitWindow *win, TickitCursorShape shape)
 {
   win->cursor.shape = shape;
 
@@ -757,7 +960,7 @@ static void _focus_lost(TickitWindow *win)
   }
 }
 
-bool tickit_window_is_focused(TickitWindow *win)
+bool tickit_window_is_focused(const TickitWindow *win)
 {
   return win->is_focused;
 }
